import py_cui
import sys
import json

from .Client import Client

class UiWidgetSetDoesNotExistError(Exception):
    pass

class UiObjectExistsError(Exception):
    pass

class UI:
    _ui_objects = {}
    _ui_widget_sets = {}

    @property
    def root(self):
        return self._root

    def add(self, name, widget):
        if name in self._ui_objects.keys():
            raise UiObjectExistsError
        self._ui_objects[name] = widget

    def get(self, name):
        return self._ui_objects[name]

    def add_screen(self, name, x=2, y=2):
        newScreen = UIScreen(self._root.create_new_widget_set(y, x), x, y)
        self._ui_widget_sets[name] = newScreen
        return self._ui_widget_sets[name]

    def activate_screen(self, name):
        if name not in self._ui_widget_sets.keys():
            raise UiWidgetSetDoesNotExistError
        self._root.apply_widget_set(self._ui_widget_sets[name]._root)

    def start(self):
        self._root.start()

    def __init__(self,
            x=2,
            y=2):
        self._root = py_cui.PyCUI(x, y)
        self._root.set_title(sys.argv[0])

class UIScreen(UI):
    def update_grid_height_width(self, height, width):
        self._root._grid.set_num_rows(height)
        self._root._grid.set_num_cols(width)

    def __init__(self,
                 uiScreen,
                 x=2,
                 y=2):
        self._root = uiScreen
        self.update_grid_height_width(y, x)

class UiWindow:
    clientType = Client
    clients = {}
    snmp_data = {}

    def __exit(self):
        sys.exit(0)

    def _printDict(self, data):
        strData = json.dumps(data, indent=2, default=str)
        retval = strData.splitlines()
        return retval

    def _setSNMP(self, clientType, host, version, userName, authPass, privPass, extraMibs):
        return self.clientType(
            host=host,
            version=3,
            userName=userName,
            authPass=authPass,
            privPass=privPass,
            extraMibs=extraMibs
            )

    def setSNMP(self, host, version, userName, authPass, privPass, extraMibs):
        self.clients[host] = self._setSNMP(
            clientType,
            host=host,
            version=3,
            userName=userName,
            authPass=authPass,
            privPass=privPass,
            extraMibs=extraMibs
            )

    def __init__(self):
        self._ui = UI(x=4, y=3)

class MenuValue:
    def __init__(self, key, val):
        self._key = key
        self._val = val

    def get(self):
        return self._val

    def __str__(self):
        return self._key



class Client:
    host = None
    community = None
    version = None
    port = None
    userName = None
    authPass = None
    privPass = None
    authData = None

    def getIfTables(self, m):
        results = {}
        for i in m.ifDescr:
            if m.ifType[i] != 6:
                continue
            if m.ifAdminStatus[i] != 1:
                continue
            results[i] = {
                    'ifDescr': m.ifDescr[i],
                    'ifType': m.ifType[i],
                    'ifSpeed': m.ifSpeed[i],
                    'ifPhysAddress': m.ifPhysAddress[i],
                    'ifAdminStatus': m.ifAdminStatus[i],
                    'ifOperStatus': m.ifOperStatus[i],
                    'ifInDiscards': m.ifInDiscards[i],
                    'ifInErrors': m.ifInErrors[i],
                    'ifOutDiscards': m.ifOutDiscards[i],
                    'ifOutErrors': m.ifOutErrors[i],
                    }
        return results



import yaml
import json

class Configuration:
    _filename = None
    ExtraMibs = []
    Host = None
    Version = None
    SecName = None
    AuthProto = None
    AuthPassword = None
    PrivProto = None
    PrivPassword = None

    def __str__(self):
        data = {
            'Host': self.Host,
            'Version': self.Version,
            'Username': self.SecName,
            'AuthProto': self.AuthProto,
            'AuthPassword': self.AuthPassword,
            'PrivProto': self.PrivProto,
            'PrivPassword': self.PrivPassword,
            'ExtraMibs': self.ExtraMibs
            }
        return json.dumps(data)

    def save(self):
        data = {
            'Host': self.Host,
            'Version': self.Version,
            'Username': self.SecName,
            'AuthProto': self.AuthProto,
            'AuthPassword': self.AuthPassword,
            'PrivProto': self.PrivProto,
            'PrivPassword': self.PrivPassword,
            'ExtraMibs': self.ExtraMibs
            }
        with open(filename, 'w') as f:
            yaml.dump(data, f)

    def _parseconfig(self, data):
        self.ExtraMibs = data.get('ExtraMibs')
        self.Host = data.get('Host')
        self.Version = data.get('Version')
        self.SecName = data.get('Username')
        self.AuthProto = data.get('AuthProto')
        self.AuthPassword = data.get('AuthPassword')
        self.PrivProto = data.get('PrivProto')
        self.PrivPassword = data.get('PrivPassword')

    def __init__(self, filename):
        data = None
        self._filename = filename
        with open(filename, 'r') as f:
            data = yaml.safe_load(f)
            self._parseconfig(data)



from .WindowClient import WindowClient
from .ui import UiWindow, MenuValue
import ipaddress
import sys
import snimpy

class CdpClient(WindowClient):
    CdpCapabilities = {
            0b1000000: "Repeater",
            0b100000: "IGMP Capable",
            0b10000: "Host",
            0b1000: "Switch",
            0b100: "Source Route Bridge",
            0b10: "Transparent Bridge",
            0b1: "Router"
            }

    def parseCapabilities(self, val):
        retval = {}
        retval['decoded'] = []
        for cap, capVal in self.CdpCapabilities.items():
            if val & cap:
                retval['decoded'].append(capVal)
        remainder = "{:025b}".format(val >> 7)
        retval['remainder'] = remainder
        return retval

    def getCdpCacheTable(self):
        results = {}
        m = self.M
        for i in m.cdpCacheAddress:
            result = {
                    'ifDescr': self._doTryGet("ifDescr", i[0]),
                    'cdpCacheAddress': self._doTryGet("cdpCacheAddress", i),
                    'cdpCacheVersion': self._doTryGet("cdpCacheVersion", i),
                    'cdpCacheDeviceId': self._doTryGet("cdpCacheDeviceId", i),
                    'cdpCacheDevicePort': self._doTryGet("cdpCacheDevicePort", i),
                    'cdpCachePlatform': self._doTryGet("cdpCachePlatform", i),
                    'cdpCacheCapabilities': self.parseCapabilities(int.from_bytes(self._doTryGet("cdpCacheCapabilities", i), "big")),
                    'cdpCacheVTPMgmtDomain': self._doTryGet("cdpCacheVTPMgmtDomain", i),
                    'cdpCacheNativeVLAN': self._doTryGet("cdpCacheNativeVLAN", i),
                    'cdpCacheDuplex': self._doTryGet("cdpCacheDuplex", i),
                    'cdpCacheApplianceID': self._doTryGet("cdpCacheApplianceID", i),
                    'cdpCacheVlanID': self._doTryGet("cdpCacheVlanID", i),
                    'cdpCachePowerConsumption': self._doTryGet("cdpCachePowerConsumption", i),
                    'cdpCacheMTU': self._doTryGet("cdpCacheMTU", i),
                    'cdpCacheSysName': self._doTryGet("cdpCacheSysName", i),
                    'cdpCacheSysObjectID': self._doTryGet("cdpCacheSysObjectID", i),
                    'cdpCachePrimaryMgmtAddrType': self._doTryGet("cdpCachePrimaryMgmtAddrType", i),
                    'cdpCachePrimaryMgmtAddr': self._doTryGet("cdpCachePrimaryMgmtAddr", i),
                    'cdpCacheSecondaryMgmtAddrType': self._doTryGet("cdpCacheSecondaryMgmtAddrType", i),
                    'cdpCacheSecondaryMgmtAddr': self._doTryGet("cdpCacheSecondaryMgmtAddr", i),
                    'cdpCachePhysLocation': self._doTryGet("cdpCachePhysLocation", i),
                    'cdpCacheLastChange': self._doTryGet("cdpCacheLastChange", i)
                    }
            try:
                ipAddress = ipaddress.ip_address(result['cdpCacheAddress'])
                result['cdpCacheAddress'] = ipAddress
            except Exception as e:
                print(f"ipAddress Error: {e}")
                pass
            if result['ifDescr'] not in results.keys():
                results[result['ifDescr']] = {}
            results[result['ifDescr']][i[1]] = result
        return results

    def getCieIfStatusListTable(self, m, i):
        results = {}
        operMode = m.cieInterfacesOperMode[1]
        results['cieInterfacesOperMode'] = []
        try:
            for k, v in operMode.iteritems():
                results['cieInterfacesOperMode'].append((k, v))
        except:
            pass
        results['cieInterfacesOperCause'] = m.cieInterfacesOperCause[i]
        results['cieInterfaceOwnershipBitmap'] = m.cieInterfaceOwnershipBitmap[i]
        return results

    def __init__(
            self,
            host,
            version,
            community=None,
            userName=None,
            authPass=None,
            privPass=None,
            authProtocol="SHA",
            privProtocol="AES",
            port=161,
            extraMibs=[]):
        super().__init__(
            host,
            version,
            community,
            userName,
            authPass,
            privPass,
            authProtocol,
            privProtocol,
            port,
            extraMibs)

class CdpNeighborsUI(UiWindow):
    clientType = CdpClient
    #clients = super().clients
    #snmp_data = super().snmp_data

    def __exit(self):
        sys.exit(0)

    def _onClientChanged(self):
        uxCdpInterfaces = self._ui.get('uxCdpInterfaces')
        uxCdpInterfaces.clear()
        selectedClient = self._ui.get('uxClients').get()
        for key, val in self.snmp_data[selectedClient].items():
            thisVal = MenuValue(key, val)
            uxCdpInterfaces.add_item(thisVal)
        self._onCdpInterfaceChanged()

    def _onCdpInterfaceChanged(self):
        uxCdpDetail = self._ui.get('uxCdpDetail')
        selectedInterface = self._ui.get('uxCdpInterfaces').get().get()
        uxCdpDetail.clear()
        for sKey, sVal in selectedInterface.items():
            for key, val in sVal.items():
                thisVal = ""
                if (type(val) == type(None)):
                    continue
                if (
                        (type(val) == str)
                        or (type(val) == snimpy.basictypes.String)
                        or (type(val) == ipaddress.IPv4Address)
                        or (type(val) == snimpy.basictypes.Integer)
                        or (type(val) == snimpy.basictypes.Enum)
                        or (type(val) == snimpy.basictypes.Unsigned32)
                        ):
                    firstLine = True
                    for line in str(val).splitlines():
                        if firstLine == True:
                            firstLine = False
                            thisVal = f"{key}: {str(line)}"
                            uxCdpDetail.add_item(thisVal)
                        else:
                            thisVal = f"    {str(line)}"
                            uxCdpDetail.add_item(thisVal)
                elif (type(val) == dict):
                    lines = self._printDict(val)
                    firstLine = True
                    for line in lines:
                        if firstLine == True:
                            firstLine = False
                            thisVal = f"{key}: {str(line)}"
                            uxCdpDetail.add_item(thisVal)
                        else:
                            thisVal = f"    {str(line)}"
                            uxCdpDetail.add_item(thisVal)
                else:
                    thisVal = f"{key}: {type(val)}"
                    uxCdpDetail.add_item(thisVal)
    def getSNMP(self):
        uxClients = self._ui.get("uxClients")
        uxClients.clear()
        for client, m in self.clients.items():
            self.snmp_data[client] = m.getCdpCacheTable()
            uxClients.add_item(client)
        self._onClientChanged()
        return self.snmp_data

    def setSNMP(self, host, version, userName, authPass, privPass, extraMibs):
        self.clients[host] = super()._setSNMP(self.clientType, host, version, userName, authPass, privPass, extraMibs)

    def run(self):
        self.getSNMP()

    def __init__(self, thisUI):
        self._ui = thisUI
        self._ui.update_grid_height_width(height=4, width=3)
        self._ui.add("uxExitLabel", self._ui.root.add_button('Exit', 3, 0, command=self.__exit))

        uxClients = self._ui.root.add_scroll_menu('Clients', 0, 0)
        uxClients.set_on_selection_change_event(self._onClientChanged)
        self._ui.add("uxClients", uxClients)

        uxCdpInterfaces = self._ui.root.add_scroll_menu('CDP Neighbors', 0, 1)
        uxCdpInterfaces.set_on_selection_change_event(self._onCdpInterfaceChanged)
        self._ui.add("uxCdpInterfaces", uxCdpInterfaces)

        uxCdpDetail = self._ui.root.add_scroll_menu('Neighbor', 1, 1, column_span=2, row_span=2)
        self._ui.add("uxCdpDetail", uxCdpDetail)



#!/usr/bin/env python3
from .snmp import SnmpUI
import argparse
import sys

def NotApplicationError(Exception):
    pass

def Exit():
    sys.exit(0)

def main(
        host,
        userName,
        authPass,
        privPass,
        version=3,
        extraMibs=[]
        ):
    myUI = SnmpUI(
        host,
        version,
        userName,
        authPass,
        privPass,
        extraMibs)
    myUI.run()

if __name__ == "__main__":
    raise NotApplicationError



#!/usr/bin/env python3
from . import main
from .Configuration import Configuration
import argparse

class ArgumentError(Exception):
    pass

def console_main():
    parser = argparse.ArgumentParser(
            description = "Python SNMP TUI (Terminal User Interface)"
            )
    parser.add_argument('-H', '--host', help="SNMP Host")
    parser.add_argument('-u', '--user', help="SNMP Username")
    parser.add_argument('-p', '--authpass', help="SNMP Auth Password")
    parser.add_argument('-x', '--privpass', help="SNMP Encryption (priv) Password")
    parser.add_argument('-v', '--version', choices=["3"], default="3", help="SNMP Version")
    parser.add_argument('-c', '--config', help="Configuration file")
    args = parser.parse_args()
    config = None
    host = None
    version = None
    userName = None
    authPass = None
    privPass = None
    extraMibs = []

    if args.config != None:
        config = Configuration(args.config)
        host = config.Host
        version = config.Version
        userName = config.SecName
        authPass = config.AuthPassword
        privPass = config.PrivPassword
        extraMibs = config.ExtraMibs

    print(args.config)
    print(config)
    print(host)
    print(version)

    if (host == None) and (args.host != None):
        host = args.host
    elif (host == None) and (args.host == None):
        raise ArgumentError
    else:
        pass

    if (host == None) and (args.user != None):
        userName = args.SecName
    elif (userName == None) and (args.SecName == None):
        raise ArgumentError
    else:
        pass

    if (authPass == None) and (args.authpass != None):
        authPass = args.authpass
    elif (authPass == None) and (args.authpass == None):
        raise ArgumentError
    else:
        pass

    if (privPass == None) and (args.privpass != None):
        privPass = args.privpass
    elif (privPass == None) and (args.privpass == None):
        raise ArgumentError
    else:
        pass

    if (version == None):
        version = args.version

    if version == "3":
        version = 3
    main(
        host,
        userName,
        authPass,
        privPass,
        version,
        extraMibs)

if __name__ == "__main__":
    console_main()




import sys
import ipaddress
import json
import snimpy
import yaml
from snimpy.config import Conf
from snimpy import manager

# Local Imports
from .ui import UI
# SNMP Client Modules
from .CdpNeighbors import CdpClient, CdpNeighborsUI

class SnmpError(Exception):
    pass

class SnmpVersionError(Exception):
    pass

class DeadCodeClass:
    def getNetTables(self):
        results = {}
        m = self.M
        results['ifTable'] = self.getIfTables(m)
        return results

    def __init__(self,
            host,
            version,
            community=None,
            userName=None,
            authPass=None,
            privPass=None,
            privProtocol="AES",
            authProtocol="SHA",
            port=161):
        self.host = host
        self.community = community
        self.version = version
        self.port = port
        manager.load("IF-MIB")
        manager.load("ENTITY-MIB")
        manager.load("/home/michael/.snmp/mibs/CISCO-SMI")
        manager.load("/home/michael/.snmp/mibs/CISCO-TC")
        manager.load("/home/michael/.snmp/mibs/CISCO-IF-EXTENSION-MIB")
        manager.load("/home/michael/.snmp/mibs/CISCO-VTP-MIB")
        manager.load("/home/michael/.snmp/mibs/CISCO-CDP-MIB")
        self.M = manager.Manager(
                host=host,
                version=version,
                secname=userName,
                authprotocol=authProtocol,
                authpassword=authPass,
                privprotocol=privProtocol,
                privpassword=privPass)

class SnmpUI:
    def __exit(self):
        sys.exit(0)

    def run(self):
        self._ui.start()

    def __init__(
            self,
            host,
            version,
            userName,
            authPass,
            privPass,
            extraMibs
            ):
        self._ui = UI()
        cdpScreen = self._ui.add_screen("CDP")
        cdpUI = CdpNeighborsUI(cdpScreen)
        cdpUI.setSNMP(
            host=host,
            version=version,
            userName=userName,
            authPass=authPass,
            privPass=privPass,
            extraMibs=extraMibs)
        cdpUI.run()
        self._ui.activate_screen("CDP")



from snimpy import manager
import sys
import json

class WindowClient:
    # M will become the SNMP Manager provided by snimpy
    M = None

    host = None
    community = None
    version = None
    port = 161

    def _doTryGet(self, func, i):
        m = self.M
        try:
            return m.__getattribute__(func)[i]
        except Exception as e:
            print(e)
            return None

    def __init__(self,
                 host,
                 version,
                 community=None,
                 userName=None,
                 authPass=None,
                 privPass=None,
                 authProtocol="SHA",
                 privProtocol="AES",
                 port=161,
                 extraMibs=[]
                 ):
        self.host = host
        self.community = community
        self.version = version
        self.port = port
        print(f"Extra MIBs: {json.dumps(extraMibs)}", file=sys.stderr)
        for mib in extraMibs:
            print(f"Loading: {mib}", file=sys.stderr)
            manager.load(mib)
        self.M = manager.Manager(
            host=host,
            version=version,
            secname=userName,
            authprotocol=authProtocol,
            authpassword=authPass,
            privprotocol=privProtocol,
            privpassword=privPass
            )
